@@ -60,8 +60,7 @@ typedef void (*ofono_radio_settings_rat_mode_set_cb_t)(
 						void *data);
 typedef void (*ofono_radio_settings_rat_mode_query_cb_t)(
 					const struct ofono_error *error,
-					enum ofono_radio_access_mode mode,
-					void *data);
+					int mode, void *data);
 
 typedef void (*ofono_radio_settings_band_set_cb_t)(
 						const struct ofono_error *error,
@@ -86,15 +85,13 @@ typedef void (*ofono_radio_settings_available_rats_query_cb_t)(
 						void *data);
 
 struct ofono_radio_settings_driver {
-	const char *name;
 	int (*probe)(struct ofono_radio_settings *rs, unsigned int vendor,
 			void *data);
 	void (*remove)(struct ofono_radio_settings *rs);
 	void (*query_rat_mode)(struct ofono_radio_settings *rs,
 				ofono_radio_settings_rat_mode_query_cb_t cb,
 				void *data);
-	void (*set_rat_mode)(struct ofono_radio_settings *rs,
-				enum ofono_radio_access_mode mode,
+	void (*set_rat_mode)(struct ofono_radio_settings *rs, unsigned int mode,
 				ofono_radio_settings_rat_mode_set_cb_t cb,
 				void *data);
 	void (*query_band)(struct ofono_radio_settings *rs,
@@ -117,11 +114,6 @@ struct ofono_radio_settings_driver {
 			void *data);
 };
 
-int ofono_radio_settings_driver_register(
-				const struct ofono_radio_settings_driver *d);
-void ofono_radio_settings_driver_unregister(
-				const struct ofono_radio_settings_driver *d);
-
 struct ofono_radio_settings *ofono_radio_settings_create(
 						struct ofono_modem *modem,
 						unsigned int vendor,
@@ -133,6 +125,9 @@ void ofono_radio_settings_remove(struct ofono_radio_settings *rs);
 void ofono_radio_settings_set_data(struct ofono_radio_settings *rs, void *data);
 void *ofono_radio_settings_get_data(struct ofono_radio_settings *rs);
 
+struct ofono_modem *ofono_radio_settings_get_modem(
+					struct ofono_radio_settings *rs);
+
 #ifdef __cplusplus
 }
 #endif
