@@ -32,6 +32,7 @@
 #include <libudev.h>
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 #include <ofono/plugin.h>
@@ -53,27 +54,27 @@ struct modem_info {
 	enum modem_type type;
 	union {
 		GSList *devices;
-		struct serial_device_info* serial;
+		struct serial_device_info *serial;
 	};
 	struct ofono_modem *modem;
 	const char *sysattr;
 };
 
 struct device_info {
-	char *devpath;
 	char *devnode;
 	char *interface;
 	char *number;
 	char *label;
 	char *sysattr;
-	char *subsystem;
+	char *kernel_driver;
+	struct udev_device *udev_device;
 };
 
 struct serial_device_info {
 	char *devpath;
 	char *devnode;
 	char *subsystem;
-	struct udev_device* dev;
+	struct udev_device *dev;
 };
 
 static gboolean setup_isi(struct modem_info *modem)
@@ -195,26 +196,76 @@ static gboolean setup_hso(struct modem_info *modem)
 	return TRUE;
 }
 
+static int setup_qmi(struct modem_info *modem, const struct device_info *qmi,
+			const struct device_info *net)
+{
+	const char *attr_value;
+	uint32_t ifindex;
+	int r;
+
+	DBG("qmi: %s net: %s kernel_driver: %s interface_number: %s",
+		qmi->devnode, net->devnode, net->kernel_driver, net->number);
+
+	if (!qmi->kernel_driver || !net->number)
+		return -EINVAL;
+
+	attr_value = udev_device_get_sysattr_value(net->udev_device,
+								"ifindex");
+	if (!attr_value)
+		return -EINVAL;
+
+	r = l_safe_atou32(attr_value, &ifindex);
+	if (r < 0)
+		return r;
+
+	ofono_modem_set_driver(modem->modem, "gobi");
+	ofono_modem_set_string(modem->modem, "Device", qmi->devnode);
+	ofono_modem_set_string(modem->modem, "KernelDriver",
+							net->kernel_driver);
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net->devnode);
+	ofono_modem_set_string(modem->modem, "InterfaceNumber", net->number);
+	ofono_modem_set_integer(modem->modem, "NetworkInterfaceIndex",
+							ifindex);
+
+	switch (modem->type) {
+	case MODEM_TYPE_USB:
+		ofono_modem_set_string(modem->modem, "Bus", "usb");
+		break;
+	case MODEM_TYPE_PCIE:
+		ofono_modem_set_string(modem->modem, "Bus", "pcie");
+		break;
+	case MODEM_TYPE_SERIAL:
+		break;
+	}
+
+	return 0;
+}
+
 static gboolean setup_gobi(struct modem_info *modem)
 {
-	const char *qmi = NULL, *mdm = NULL, *net = NULL;
-	const char *gps = NULL, *diag = NULL;
+	const struct device_info *qmi = NULL;
+	const struct device_info *net = NULL;
+	const char *mdm = NULL;
+	const char *gps = NULL;
+	const char *diag = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = list->next) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s %s", info->devnode, info->interface,
 						info->number, info->label,
-						info->sysattr, info->subsystem);
+						info->sysattr, subsystem);
 
-		if (g_strcmp0(info->subsystem, "usbmisc") == 0) /* cdc-wdm */
-			qmi = info->devnode;
-		else if (g_strcmp0(info->subsystem, "net") == 0) /* wwan */
-			net = info->devnode;
-		else if (g_strcmp0(info->subsystem, "tty") == 0) {
+		if (g_strcmp0(subsystem, "usbmisc") == 0) /* cdc-wdm */
+			qmi = info;
+		else if (g_strcmp0(subsystem, "net") == 0) /* wwan */
+			net = info;
+		else if (g_strcmp0(subsystem, "tty") == 0) {
 			if (g_strcmp0(info->interface, "255/255/255") == 0) {
 				if (g_strcmp0(info->number, "00") == 0)
 					diag = info->devnode; /* ec20 */
@@ -234,32 +285,40 @@ static gboolean setup_gobi(struct modem_info *modem)
 		}
 	}
 
-	DBG("qmi=%s net=%s mdm=%s gps=%s diag=%s", qmi, net, mdm, gps, diag);
-
 	if (qmi == NULL || mdm == NULL || net == NULL)
 		return FALSE;
 
+	DBG("qmi=%s net=%s mdm=%s gps=%s diag=%s",
+			qmi->devnode, net->devnode, mdm, gps, diag);
+
+	if (setup_qmi(modem, qmi, net) < 0)
+		return FALSE;
 
-	ofono_modem_set_string(modem->modem, "Device", qmi);
 	ofono_modem_set_string(modem->modem, "Modem", mdm);
 	ofono_modem_set_string(modem->modem, "Diag", diag);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
 
 static gboolean setup_sierra(struct modem_info *modem)
 {
-	const char *mdm = NULL, *app = NULL, *net = NULL, *diag = NULL, *qmi = NULL;
+	const struct device_info *net = NULL;
+	const struct device_info *qmi = NULL;
+	const char *mdm = NULL;
+	const char *app = NULL;
+	const char *diag = NULL;
+
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = list->next) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
-				info->number, info->label, info->subsystem);
+				info->number, info->label, subsystem);
 
 		if (g_strcmp0(info->interface, "255/255/255") == 0) {
 			if (g_strcmp0(info->number, "01") == 0)
@@ -269,8 +328,8 @@ static gboolean setup_sierra(struct modem_info *modem)
 			else if (g_strcmp0(info->number, "04") == 0)
 				app = info->devnode;
 			else if (g_strcmp0(info->number, "07") == 0)
-				net = info->devnode;
-			else if (g_strcmp0(info->subsystem, "net") == 0) {
+				net = info;
+			else if (g_strcmp0(subsystem, "net") == 0) {
 				/*
 				 * When using the voice firmware on a mc7304
 				 * the second cdc-wdm interface doesn't handle
@@ -280,52 +339,58 @@ static gboolean setup_sierra(struct modem_info *modem)
 				 * the first interface works.
 				 */
 				if (g_strcmp0(info->number, "08") == 0) {
-					net = info->devnode;
+					net = info;
 				} else if (g_strcmp0(info->number, "0a") == 0) {
 					if (net == NULL)
-						net = info->devnode;
+						net = info;
 				}
-			} else if (g_strcmp0(info->subsystem, "usbmisc") == 0) {
+			} else if (g_strcmp0(subsystem, "usbmisc") == 0) {
 				if (g_strcmp0(info->number, "08") == 0) {
-					qmi = info->devnode;
+					qmi = info;
 				} else if (g_strcmp0(info->number, "0a") == 0) {
 					if (qmi == NULL)
-						qmi = info->devnode;
+						qmi = info;
 				}
 			}
 		}
 	}
 
 	if (qmi != NULL && net != NULL) {
-		ofono_modem_set_driver(modem->modem, "gobi");
+		if (setup_qmi(modem, qmi, net) < 0)
+			return FALSE;
+
 		goto done;
 	}
 
 	if (mdm == NULL || net == NULL)
 		return FALSE;
 
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net->devnode);
 done:
-	DBG("modem=%s app=%s net=%s diag=%s qmi=%s", mdm, app, net, diag, qmi);
+	DBG("modem=%s app=%s net=%s diag=%s qmi=%s",
+			mdm, app, net->devnode, diag, qmi->devnode);
 
-	ofono_modem_set_string(modem->modem, "Device", qmi);
 	ofono_modem_set_string(modem->modem, "Modem", mdm);
 	ofono_modem_set_string(modem->modem, "App", app);
 	ofono_modem_set_string(modem->modem, "Diag", diag);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
 
 static gboolean setup_huawei(struct modem_info *modem)
 {
-	const char *qmi = NULL, *mdm = NULL, *net = NULL;
-	const char *pcui = NULL, *diag = NULL;
+	const struct device_info *net = NULL;
+	const struct device_info *qmi = NULL;
+	const char *mdm = NULL;
+	const char *pcui = NULL;
+	const char *diag = NULL;
+
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = list->next) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
 
 		DBG("%s %s %s %s", info->devnode, info->interface,
 						info->number, info->label);
@@ -350,10 +415,10 @@ static gboolean setup_huawei(struct modem_info *modem)
 			diag = info->devnode;
 		} else if (g_strcmp0(info->interface, "255/1/8") == 0 ||
 				g_strcmp0(info->interface, "255/1/56") == 0) {
-			net = info->devnode;
+			net = info;
 		} else if (g_strcmp0(info->interface, "255/1/9") == 0 ||
 				g_strcmp0(info->interface, "255/1/57") == 0) {
-			qmi = info->devnode;
+			qmi = info;
 		} else if (g_strcmp0(info->interface, "255/255/255") == 0) {
 			if (g_strcmp0(info->number, "00") == 0)
 				mdm = info->devnode;
@@ -369,21 +434,23 @@ static gboolean setup_huawei(struct modem_info *modem)
 	}
 
 	if (qmi != NULL && net != NULL) {
-		ofono_modem_set_driver(modem->modem, "gobi");
+		if (setup_qmi(modem, qmi, net) < 0)
+			return FALSE;
+
 		goto done;
 	}
 
 	if (mdm == NULL || pcui == NULL)
 		return FALSE;
 
+	ofono_modem_set_string(modem->modem, "NetworkInterface", net->devnode);
 done:
-	DBG("mdm=%s pcui=%s diag=%s qmi=%s net=%s", mdm, pcui, diag, qmi, net);
+	DBG("mdm=%s pcui=%s diag=%s qmi=%s net=%s",
+		mdm, pcui, diag, qmi->devnode, net->devnode);
 
-	ofono_modem_set_string(modem->modem, "Device", qmi);
 	ofono_modem_set_string(modem->modem, "Modem", mdm);
 	ofono_modem_set_string(modem->modem, "Pcui", pcui);
 	ofono_modem_set_string(modem->modem, "Diag", diag);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
 	return TRUE;
 }
@@ -677,37 +744,67 @@ static gboolean setup_telit(struct modem_info *modem)
 
 static gboolean setup_telitqmi(struct modem_info *modem)
 {
-	const char *qmi = NULL, *net = NULL;
+	const struct device_info *net = NULL;
+	const struct device_info *qmi = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = list->next) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
+		const char *subsystem =
+				udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
-				info->number, info->label, info->subsystem);
+				info->number, info->label, subsystem);
 
 		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
 				g_strcmp0(info->number, "02") == 0) {
-			if (g_strcmp0(info->subsystem, "net") == 0)
-				net = info->devnode;
-			else if (g_strcmp0(info->subsystem, "usbmisc") == 0)
-				qmi = info->devnode;
+			if (g_strcmp0(subsystem, "net") == 0)
+				net = info;
+			else if (g_strcmp0(subsystem, "usbmisc") == 0)
+				qmi = info;
 		}
 	}
 
 	if (qmi == NULL || net == NULL)
 		return FALSE;
 
-	DBG("qmi=%s net=%s", qmi, net);
-
-	ofono_modem_set_string(modem->modem, "Device", qmi);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
+	if (setup_qmi(modem, qmi, net) < 0)
+		return FALSE;
 
 	ofono_modem_set_boolean(modem->modem, "ForceSimLegacy", TRUE);
 	ofono_modem_set_boolean(modem->modem, "AlwaysOnline", TRUE);
-	ofono_modem_set_driver(modem->modem, "gobi");
+
+	return TRUE;
+}
+
+static gboolean setup_droid(struct modem_info *modem)
+{
+	const char *at = NULL;
+	GSList *list;
+
+	DBG("%s", modem->syspath);
+
+	for (list = modem->devices; list; list = list->next) {
+		struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
+
+		DBG("%s %s %s %s %s", info->devnode, info->interface,
+				info->number, info->label, subsystem);
+
+		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
+				g_strcmp0(info->number, "04") == 0) {
+			at = info->devnode;
+		}
+	}
+
+	if (at == NULL)
+		return FALSE;
+
+	ofono_modem_set_string(modem->modem, "Device", at);
+	ofono_modem_set_driver(modem->modem, "droid");
 
 	return TRUE;
 }
@@ -893,6 +990,11 @@ static gboolean setup_quectel_serial(struct modem_info *modem)
 	if (value)
 		ofono_modem_set_string(modem->modem, "GpioOffset", value);
 
+	value = udev_device_get_property_value(info->dev,
+						"OFONO_QUECTEL_GPIO_LEVEL");
+	if (value)
+		ofono_modem_set_boolean(modem->modem, "GpioLevel", TRUE);
+
 	value = udev_device_get_property_value(info->dev,
 						"OFONO_QUECTEL_MUX");
 	if (value)
@@ -908,31 +1010,66 @@ static gboolean setup_quectel_serial(struct modem_info *modem)
 
 static gboolean setup_quectel(struct modem_info *modem)
 {
-	if (modem->serial)
+	if (modem->type == MODEM_TYPE_SERIAL)
 		return setup_quectel_serial(modem);
-	else
+	else if (modem->type == MODEM_TYPE_USB)
 		return setup_quectel_usb(modem);
+	else
+		return FALSE;
+}
+
+static gboolean is_premultiplexed(const struct device_info *net)
+{
+	struct udev_device *parent = udev_device_get_parent(net->udev_device);
+
+	if (!parent)
+		return FALSE;
+
+	if (g_strcmp0(udev_device_get_subsystem(parent), "net") == 0)
+		return TRUE;
+
+	return FALSE;
 }
 
 static gboolean setup_quectelqmi(struct modem_info *modem)
 {
-	const char *qmi = NULL, *net = NULL, *gps = NULL, *aux = NULL;
+	const struct device_info *net = NULL;
+	const struct device_info *qmi = NULL;
+	const char *gps = NULL;
+	const char *aux = NULL;
 	GSList *list;
+	const char *premux_interfaces[8];
+	int n_premux = 0;
+	const char *qmap_size;
+
+	memset(premux_interfaces, 0, sizeof(premux_interfaces));
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = g_slist_next(list)) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
-				info->number, info->label, info->subsystem);
+				info->number, info->label, subsystem);
 
-		if (g_strcmp0(info->interface, "255/255/255") == 0 &&
-				g_strcmp0(info->number, "04") == 0) {
-			if (g_strcmp0(info->subsystem, "net") == 0)
-				net = info->devnode;
-			else if (g_strcmp0(info->subsystem, "usbmisc") == 0)
-				qmi = info->devnode;
+		if (g_strcmp0(info->interface, "255/255/255") == 0) {
+			if (g_strcmp0(subsystem, "usbmisc") == 0) {
+				qmi = info;
+				continue;
+			}
+
+			if (g_strcmp0(subsystem, "net"))
+				continue;
+
+			if (is_premultiplexed(info)) {
+				premux_interfaces[n_premux] = info->devnode;
+				n_premux += 1;
+				continue;
+			}
+
+			net = info;
 		} else if (g_strcmp0(info->interface, "255/0/0") == 0 &&
 				g_strcmp0(info->number, "01") == 0) {
 			gps = info->devnode;
@@ -942,24 +1079,51 @@ static gboolean setup_quectelqmi(struct modem_info *modem)
 		}
 	}
 
-	DBG("qmi=%s net=%s", qmi, net);
-
 	if (qmi == NULL || net == NULL)
 		return FALSE;
 
-	DBG("qmi=%s net=%s", qmi, net);
-
-	ofono_modem_set_string(modem->modem, "Device", qmi);
-	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
-
 	DBG("gps=%s aux=%s", gps, aux);
 
+	if (setup_qmi(modem, qmi, net) < 0)
+		return FALSE;
+
+	qmap_size = udev_device_get_sysattr_value(net->udev_device,
+							"qmap_size");
+	if (qmap_size) {
+		uint32_t max_aggregation_size;
+
+		if (l_safe_atou32(qmap_size, &max_aggregation_size) == 0)
+			ofono_modem_set_integer(modem->modem,
+						"MaxAggregationSize",
+						max_aggregation_size);
+	}
+
 	if (gps)
 		ofono_modem_set_string(modem->modem, "GPS", gps);
+
 	if (aux)
 		ofono_modem_set_string(modem->modem, "Aux", aux);
 
-	ofono_modem_set_driver(modem->modem, "gobi");
+	if (n_premux) {
+		char buf[256];
+		int i;
+
+		ofono_modem_set_integer(modem->modem,
+					"NumPremuxInterfaces", n_premux);
+		for (i = 0; i < n_premux; i++) {
+			const char *device = premux_interfaces[i];
+			int len = strlen(device);
+
+			if (!len)
+				continue;
+
+			sprintf(buf, "PremuxInterface%d", i + 1);
+			ofono_modem_set_string(modem->modem, buf, device);
+			sprintf(buf, "PremuxInterface%dMuxId", i + 1);
+			ofono_modem_set_integer(modem->modem, buf,
+						0x80 + device[len - 1] - '0');
+		}
+	}
 
 	return TRUE;
 }
@@ -974,16 +1138,18 @@ static gboolean setup_mbim(struct modem_info *modem)
 
 	for (list = modem->devices; list; list = list->next) {
 		struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s %s", info->devnode, info->interface,
 						info->number, info->label,
-						info->sysattr, info->subsystem);
+						info->sysattr, subsystem);
 
-		if (g_strcmp0(info->subsystem, "usbmisc") == 0) /* cdc-wdm */
+		if (g_strcmp0(subsystem, "usbmisc") == 0) /* cdc-wdm */
 			ctl = info->devnode;
-		else if (g_strcmp0(info->subsystem, "net") == 0) /* wwan */
+		else if (g_strcmp0(subsystem, "net") == 0) /* wwan */
 			net = info->devnode;
-		else if (g_strcmp0(info->subsystem, "tty") == 0) {
+		else if (g_strcmp0(subsystem, "tty") == 0) {
 			if (g_strcmp0(info->number, "02") == 0)
 				atcmd = info->devnode;
 		}
@@ -1003,9 +1169,9 @@ static gboolean setup_mbim(struct modem_info *modem)
 	return TRUE;
 }
 
-static gboolean setup_serial_modem(struct modem_info* modem)
+static gboolean setup_serial_modem(struct modem_info *modem)
 {
-	struct serial_device_info* info;
+	struct serial_device_info *info;
 
 	info = modem->serial;
 
@@ -1014,23 +1180,23 @@ static gboolean setup_serial_modem(struct modem_info* modem)
 	return TRUE;
 }
 
-static gboolean setup_tc65(struct modem_info* modem)
+static gboolean setup_tc65(struct modem_info *modem)
 {
 	ofono_modem_set_driver(modem->modem, "cinterion");
 
 	return setup_serial_modem(modem);
 }
 
-static gboolean setup_ehs6(struct modem_info* modem)
+static gboolean setup_ehs6(struct modem_info *modem)
 {
 	ofono_modem_set_driver(modem->modem, "cinterion");
 
 	return setup_serial_modem(modem);
 }
 
-static gboolean setup_ifx(struct modem_info* modem)
+static gboolean setup_ifx(struct modem_info *modem)
 {
-	struct serial_device_info* info;
+	struct serial_device_info *info;
 	const char *value;
 
 	info = modem->serial;
@@ -1052,9 +1218,9 @@ static gboolean setup_ifx(struct modem_info* modem)
 	return TRUE;
 }
 
-static gboolean setup_wavecom(struct modem_info* modem)
+static gboolean setup_wavecom(struct modem_info *modem)
 {
-	struct serial_device_info* info;
+	struct serial_device_info *info;
 	const char *value;
 
 	info = modem->serial;
@@ -1069,9 +1235,9 @@ static gboolean setup_wavecom(struct modem_info* modem)
 	return TRUE;
 }
 
-static gboolean setup_isi_serial(struct modem_info* modem)
+static gboolean setup_isi_serial(struct modem_info *modem)
 {
-	struct serial_device_info* info;
+	struct serial_device_info *info;
 	const char *value;
 
 	info = modem->serial;
@@ -1159,10 +1325,10 @@ static gboolean setup_ublox(struct modem_info *modem)
 	return TRUE;
 }
 
-static gboolean setup_gemalto(struct modem_info* modem)
+static gboolean setup_gemalto(struct modem_info *modem)
 {
 	const char *app = NULL, *gps = NULL, *mdm = NULL,
-		*net = NULL, *qmi = NULL;
+		*net = NULL, *qmi = NULL, *net2 = NULL;
 
 	GSList *list;
 
@@ -1170,9 +1336,11 @@ static gboolean setup_gemalto(struct modem_info* modem)
 
 	for (list = modem->devices; list; list = list->next) {
 		struct device_info *info = list->data;
+		const char *subsystem =
+			udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s", info->devnode, info->interface,
-				info->number, info->label, info->subsystem);
+				info->number, info->label, subsystem);
 
 		/* PHS8-P */
 		if (g_strcmp0(info->interface, "255/255/255") == 0) {
@@ -1182,13 +1350,13 @@ static gboolean setup_gemalto(struct modem_info* modem)
 				app = info->devnode;
 			else if (g_strcmp0(info->number, "03") == 0)
 				mdm = info->devnode;
-			else if (g_strcmp0(info->subsystem, "net") == 0)
+			else if (g_strcmp0(subsystem, "net") == 0)
 				net = info->devnode;
-			else if (g_strcmp0(info->subsystem, "usbmisc") == 0)
+			else if (g_strcmp0(subsystem, "usbmisc") == 0)
 				qmi = info->devnode;
 		}
 
-		/* Cinterion ALS3, PLS8-E, PLS8-X */
+		/* Cinterion ALS3, PLS8-E, PLS8-X, EHS5-E */
 		if (g_strcmp0(info->interface, "2/2/1") == 0) {
 			if (g_strcmp0(info->number, "00") == 0)
 				mdm = info->devnode;
@@ -1197,9 +1365,14 @@ static gboolean setup_gemalto(struct modem_info* modem)
 			else if (g_strcmp0(info->number, "04") == 0)
 				gps = info->devnode;
 		}
+
 		if (g_strcmp0(info->interface, "2/6/0") == 0) {
-			if (g_strcmp0(info->subsystem, "net") == 0)
-				net = info->devnode;
+			if (g_strcmp0(subsystem, "net") == 0) {
+				if (g_strcmp0(info->number, "0a") == 0)
+					net = info->devnode;
+				if (g_strcmp0(info->number, "0c") == 0)
+					net2 = info->devnode;
+			}
 		}
 	}
 
@@ -1216,6 +1389,9 @@ static gboolean setup_gemalto(struct modem_info* modem)
 	ofono_modem_set_string(modem->modem, "Model", modem->model);
 	ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 
+	if (net2)
+		ofono_modem_set_string(modem->modem, "NetworkInterface2", net2);
+
 	return TRUE;
 }
 
@@ -1224,18 +1400,21 @@ static gboolean setup_xmm7xxx(struct modem_info *modem)
 	const char *mdm = NULL, *net = NULL, *net2 = NULL, *net3 = NULL;
 	GSList *list;
 
-	DBG("%s %s\n", __DATE__, __TIME__);
 	DBG("%s %s %s %s %s %s\n", modem->syspath, modem->devname,
 		modem->driver, modem->vendor, modem->model, modem->sysattr);
 
 	for (list = modem->devices; list; list = list->next) {
 		struct device_info *info = list->data;
+		const char *syspath =
+				udev_device_get_syspath(info->udev_device);
+		const char *subsystem =
+				udev_device_get_subsystem(info->udev_device);
 
-		DBG("%s %s %s %s %s %s %s\n", info->devpath, info->devnode,
+		DBG("%s %s %s %s %s %s %s\n", syspath, info->devnode,
 				info->interface, info->number, info->label,
-				info->sysattr, info->subsystem);
+				info->sysattr, subsystem);
 
-		if (g_strcmp0(info->subsystem, "pci") == 0) {
+		if (g_strcmp0(subsystem, "pci") == 0) {
 			if ((g_strcmp0(modem->vendor, "0x8086") == 0) &&
 				(g_strcmp0(modem->model, "0x7560") == 0)) {
 				mdm = "/dev/iat";
@@ -1249,10 +1428,10 @@ static gboolean setup_xmm7xxx(struct modem_info *modem)
 			}
 		} else { /* For USB */
 			if (g_strcmp0(modem->model, "095a") == 0) {
-				if (g_strcmp0(info->subsystem, "tty") == 0) {
+				if (g_strcmp0(subsystem, "tty") == 0) {
 					if (g_strcmp0(info->number, "00") == 0)
 						mdm = info->devnode;
-				} else if (g_strcmp0(info->subsystem, "net")
+				} else if (g_strcmp0(subsystem, "net")
 									== 0) {
 					if (g_strcmp0(info->number, "06") == 0)
 						net = info->devnode;
@@ -1262,10 +1441,10 @@ static gboolean setup_xmm7xxx(struct modem_info *modem)
 						net3 = info->devnode;
 				}
 			} else {
-				if (g_strcmp0(info->subsystem, "tty") == 0) {
+				if (g_strcmp0(subsystem, "tty") == 0) {
 					if (g_strcmp0(info->number, "02") == 0)
 						mdm = info->devnode;
-				} else if (g_strcmp0(info->subsystem, "net")
+				} else if (g_strcmp0(subsystem, "net")
 									== 0) {
 					if (g_strcmp0(info->number, "00") == 0)
 						net = info->devnode;
@@ -1298,19 +1477,25 @@ static gboolean setup_xmm7xxx(struct modem_info *modem)
 
 static gboolean setup_sim7x00(struct modem_info *modem)
 {
-	const char *audio = NULL, *diag = NULL, *gps = NULL;
-	const char *mdm = NULL, *net = NULL, *ppp = NULL;
-	const char *qmi = NULL;
+	const struct device_info *net = NULL;
+	const struct device_info *qmi = NULL;
+	const char *mdm = NULL;
+	const char *ppp = NULL;
+	const char *audio = NULL;
+	const char *diag = NULL;
+	const char *gps = NULL;
 	GSList *list;
 
 	DBG("%s", modem->syspath);
 
 	for (list = modem->devices; list; list = list->next) {
-		struct device_info *info = list->data;
+		const struct device_info *info = list->data;
+		const char *subsystem =
+				udev_device_get_subsystem(info->udev_device);
 
 		DBG("%s %s %s %s %s %s", info->devnode, info->interface,
 						info->number, info->label,
-						info->sysattr, info->subsystem);
+						info->sysattr, subsystem);
 
 		/*
 		 * SIM7100 serial port layout:
@@ -1322,11 +1507,11 @@ static gboolean setup_sim7x00(struct modem_info *modem)
 		 *
 		 * -- https://www.spinics.net/lists/linux-usb/msg135728.html
 		 */
-		if (g_strcmp0(info->subsystem, "usbmisc") == 0) /* cdc-wdm */
-			qmi = info->devnode; /* SIM7600 */
-		else if (g_strcmp0(info->subsystem, "net") == 0) /* wwan */
-			net = info->devnode; /* SIM7600 */
-		else if (g_strcmp0(info->subsystem, "tty") == 0) {
+		if (g_strcmp0(subsystem, "usbmisc") == 0) /* cdc-wdm */
+			qmi = info; /* SIM7600 */
+		else if (g_strcmp0(subsystem, "net") == 0) /* wwan */
+			net = info; /* SIM7600 */
+		else if (g_strcmp0(subsystem, "tty") == 0) {
 			if (g_strcmp0(info->interface, "255/255/255") == 0) {
 				if (g_strcmp0(info->number, "00") == 0)
 					diag = info->devnode; /* SIM7x00 */
@@ -1347,14 +1532,12 @@ static gboolean setup_sim7x00(struct modem_info *modem)
 		return FALSE;
 
 	if (qmi != NULL && net != NULL) {
-		DBG("qmi=%s net=%s mdm=%s gps=%s diag=%s",
-						qmi, net, mdm, gps, diag);
+		DBG("mdm=%s gps=%s diag=%s", mdm, gps, diag);
 
-		ofono_modem_set_driver(modem->modem, "gobi");
+		if (setup_qmi(modem, qmi, net) < 0)
+			return FALSE;
 
-		ofono_modem_set_string(modem->modem, "Device", qmi);
 		ofono_modem_set_string(modem->modem, "Modem", mdm);
-		ofono_modem_set_string(modem->modem, "NetworkInterface", net);
 	} else {
 		DBG("at=%s ppp=%s gps=%s diag=%s, audio=%s",
 						mdm, ppp, gps, diag, audio);
@@ -1382,7 +1565,6 @@ static struct {
 	{ "gobi",	setup_gobi	},
 	{ "sierra",	setup_sierra	},
 	{ "huawei",	setup_huawei	},
-	{ "speedupcdma",setup_speedup	},
 	{ "speedup",	setup_speedup	},
 	{ "linktop",	setup_linktop	},
 	{ "alcatel",	setup_alcatel	},
@@ -1401,13 +1583,13 @@ static struct {
 	{ "gemalto",	setup_gemalto	},
 	{ "xmm7xxx",	setup_xmm7xxx	},
 	{ "mbim",	setup_mbim	},
+	{ "droid",	setup_droid	},
 	/* Following are non-USB modems */
 	{ "ifx",	setup_ifx		},
 	{ "u8500",	setup_isi_serial	},
 	{ "n900",	setup_isi_serial	},
 	{ "calypso",	setup_serial_modem	},
 	{ "cinterion",	setup_serial_modem	},
-	{ "nokiacdma",	setup_serial_modem	},
 	{ "sim900",	setup_serial_modem	},
 	{ "wavecom",	setup_wavecom		},
 	{ "tc65",	setup_tc65		},
@@ -1429,19 +1611,19 @@ static const char *get_sysattr(const char *driver)
 	return NULL;
 }
 
-static void device_info_free(struct device_info* info)
+static void device_info_free(struct device_info *info)
 {
-	g_free(info->devpath);
 	g_free(info->devnode);
 	g_free(info->interface);
 	g_free(info->number);
 	g_free(info->label);
 	g_free(info->sysattr);
-	g_free(info->subsystem);
+	g_free(info->kernel_driver);
+	udev_device_unref(info->udev_device);
 	g_free(info);
 }
 
-static void serial_device_info_free(struct serial_device_info* info)
+static void serial_device_info_free(struct serial_device_info *info)
 {
 	g_free(info->devpath);
 	g_free(info->devnode);
@@ -1495,8 +1677,10 @@ static gboolean check_remove(gpointer key, gpointer value, gpointer user_data)
 	case MODEM_TYPE_PCIE:
 		for (list = modem->devices; list; list = list->next) {
 			struct device_info *info = list->data;
+			const char *syspath =
+				udev_device_get_syspath(info->udev_device);
 
-			if (g_strcmp0(info->devpath, devpath) == 0)
+			if (g_strcmp0(syspath, devpath) == 0)
 				return TRUE;
 		}
 		break;
@@ -1540,9 +1724,9 @@ static gint compare_device(gconstpointer a, gconstpointer b)
  * through the device hierarchy.
  *
  */
-static struct udev_device* get_serial_modem_device(struct udev_device *dev)
+static struct udev_device *get_serial_modem_device(struct udev_device *dev)
 {
-	const char* driver;
+	const char *driver;
 
 	while (dev) {
 		driver = udev_device_get_property_value(dev, "OFONO_DRIVER");
@@ -1571,14 +1755,12 @@ static void add_serial_device(struct udev_device *dev)
 	struct modem_info *modem;
 	struct serial_device_info *info;
 	const char *subsystem;
-	struct udev_device* mdev;
-	const char* driver;
+	struct udev_device *mdev;
+	const char *driver;
 
 	mdev = get_serial_modem_device(dev);
-	if (!mdev) {
-		DBG("Device is missing required OFONO_DRIVER property");
+	if (!mdev)
 		return;
-	}
 
 	driver = udev_device_get_property_value(mdev, "OFONO_DRIVER");
 
@@ -1593,9 +1775,7 @@ static void add_serial_device(struct udev_device *dev)
 
 	modem = g_hash_table_lookup(modem_list, syspath);
 	if (modem == NULL) {
-		modem = g_try_new0(struct modem_info, 1);
-		if (modem == NULL)
-			return;
+		modem = g_new0(struct modem_info, 1);
 
 		modem->type = MODEM_TYPE_SERIAL;
 		modem->syspath = g_strdup(syspath);
@@ -1611,9 +1791,7 @@ static void add_serial_device(struct udev_device *dev)
 	DBG("%s", devpath);
 	DBG("%s (%s)", devnode, driver);
 
-	info = g_try_new0(struct serial_device_info, 1);
-	if (info == NULL)
-		return;
+	info = g_new0(struct serial_device_info, 1);
 
 	info->devpath = g_strdup(devpath);
 	info->devnode = g_strdup(devnode);
@@ -1623,36 +1801,33 @@ static void add_serial_device(struct udev_device *dev)
 	modem->serial = info;
 }
 
-static void add_device(const char *syspath, const char *devname,
-			const char *driver, const char *vendor,
-			const char *model, struct udev_device *device,
-			enum modem_type type)
+static void add_device(const char *modem_syspath, const char *modem_devname,
+			const char *modem_driver, const char *modem_vendor,
+			const char *modem_model, enum modem_type modem_type,
+			struct udev_device *device, const char *kernel_driver)
 {
 	struct udev_device *usb_interface;
-	const char *devpath, *devnode, *interface, *number;
-	const char *label, *sysattr, *subsystem;
+	const char *devnode, *interface, *number;
+	const char *label, *sysattr;
 	struct modem_info *modem;
 	struct device_info *info;
 	struct udev_device *parent;
 
-	devpath = udev_device_get_syspath(device);
-	if (devpath == NULL)
+	if (udev_device_get_syspath(device) == NULL)
 		return;
 
-	modem = g_hash_table_lookup(modem_list, syspath);
+	modem = g_hash_table_lookup(modem_list, modem_syspath);
 	if (modem == NULL) {
-		modem = g_try_new0(struct modem_info, 1);
-		if (modem == NULL)
-			return;
+		modem = g_new0(struct modem_info, 1);
 
-		modem->type = type;
-		modem->syspath = g_strdup(syspath);
-		modem->devname = g_strdup(devname);
-		modem->driver = g_strdup(driver);
-		modem->vendor = g_strdup(vendor);
-		modem->model = g_strdup(model);
+		modem->type = modem_type;
+		modem->syspath = g_strdup(modem_syspath);
+		modem->devname = g_strdup(modem_devname);
+		modem->driver = g_strdup(modem_driver);
+		modem->vendor = g_strdup(modem_vendor);
+		modem->model = g_strdup(modem_model);
 
-		modem->sysattr = get_sysattr(driver);
+		modem->sysattr = get_sysattr(modem_driver);
 
 		g_hash_table_replace(modem_list, modem->syspath, modem);
 	}
@@ -1701,29 +1876,25 @@ static void add_device(const char *syspath, const char *devname,
 		}
 	}
 
-	subsystem = udev_device_get_subsystem(device);
-
 	if (modem->sysattr != NULL)
 		sysattr = udev_device_get_sysattr_value(device, modem->sysattr);
 	else
 		sysattr = NULL;
 
-	DBG("%s", syspath);
-	DBG("%s", devpath);
-	DBG("%s (%s) %s [%s] ==> %s %s", devnode, driver,
+	DBG("modem:%s device:%s",
+			modem->syspath, udev_device_get_syspath(device));
+	DBG("%s (%s) %s [%s] ==> %s %s", devnode, modem->driver,
 					interface, number, label, sysattr);
 
-	info = g_try_new0(struct device_info, 1);
-	if (info == NULL)
-		return;
+	info = g_new0(struct device_info, 1);
 
-	info->devpath = g_strdup(devpath);
 	info->devnode = g_strdup(devnode);
 	info->interface = g_strdup(interface);
 	info->number = g_strdup(number);
 	info->label = g_strdup(label);
 	info->sysattr = g_strdup(sysattr);
-	info->subsystem = g_strdup(subsystem);
+	info->kernel_driver = g_strdup(kernel_driver);
+	info->udev_device = udev_device_ref(device);
 
 	modem->devices = g_slist_insert_sorted(modem->devices, info,
 							compare_device);
@@ -1772,7 +1943,6 @@ static struct {
 	{ "huawei",	"cdc_ether",	"12d1"		},
 	{ "huawei",	"qmi_wwan",	"12d1"		},
 	{ "huawei",	"option",	"12d1"		},
-	{ "speedupcdma","option",	"1c9e", "9e00"	},
 	{ "speedup",	"option",	"1c9e"		},
 	{ "speedup",	"option",	"2020"		},
 	{ "alcatel",	"option",	"1bbb", "0017"	},
@@ -1786,6 +1956,8 @@ static struct {
 	{ "telit",	"cdc_acm",	"1bc7", "0021"	},
 	{ "telitqmi",	"qmi_wwan",	"1bc7", "1201"	},
 	{ "telitqmi",	"option",	"1bc7", "1201"	},
+	{ "droid",	"qmi_wwan",	"22b8", "2a70"	},
+	{ "droid",	"option",	"22b8", "2a70"	},
 	{ "nokia",	"option",	"0421", "060e"	},
 	{ "nokia",	"option",	"0421", "0623"	},
 	{ "samsung",	"option",	"04e8", "6889"	},
@@ -1795,6 +1967,9 @@ static struct {
 	{ "quectelqmi",	"qcserial",	"2c7c", "0121"	},
 	{ "quectelqmi",	"qmi_wwan",	"2c7c", "0125"	},
 	{ "quectelqmi",	"qcserial",	"2c7c", "0125"	},
+	{ "quectelqmi",	"qmi_wwan",	"2c7c", "0296"	},
+	{ "quectelqmi",	"qcserial",	"2c7c", "0296"	},
+	{ "quectelqmi", "qmi_wwan_q",	"2c7c", "0452"	},
 	{ "ublox",	"cdc_acm",	"1546", "1010"	},
 	{ "ublox",	"cdc_ncm",	"1546", "1010"	},
 	{ "ublox",	"cdc_acm",	"1546", "1102"	},
@@ -1805,8 +1980,11 @@ static struct {
 	{ "gemalto",	"option",	"1e2d",	"0053"	},
 	{ "gemalto",	"cdc_wdm",	"1e2d",	"0053"	},
 	{ "gemalto",	"qmi_wwan",	"1e2d",	"0053"	},
+	{ "gemalto",	"cdc_acm",	"1e2d",	"0058"	},
 	{ "gemalto",	"cdc_acm",	"1e2d",	"0061"	},
 	{ "gemalto",	"cdc_ether",	"1e2d",	"0061"	},
+	{ "gemalto",	"cdc_acm",	"1e2d",	"005b"	},
+	{ "gemalto",	"cdc_ether",	"1e2d",	"005b"	},
 	{ "telit",	"cdc_ncm",	"1bc7", "0036"	},
 	{ "telit",	"cdc_acm",	"1bc7", "0036"	},
 	{ "xmm7xxx",	"cdc_acm",	"8087"		},
@@ -1819,6 +1997,7 @@ static void check_usb_device(struct udev_device *device)
 	struct udev_device *usb_device;
 	const char *syspath, *devname, *driver;
 	const char *vendor = NULL, *model = NULL;
+	const char *kernel_driver;
 
 	usb_device = udev_device_get_parent_with_subsystem_devtype(device,
 							"usb", "usb_device");
@@ -1847,34 +2026,32 @@ static void check_usb_device(struct udev_device *device)
 					usb_interface, "OFONO_DRIVER");
 	}
 
+	kernel_driver = udev_device_get_property_value(device, "ID_USB_DRIVER");
+	if (kernel_driver == NULL) {
+		kernel_driver = udev_device_get_driver(device);
+		if (kernel_driver == NULL) {
+			struct udev_device *parent;
+
+			parent = udev_device_get_parent(device);
+			if (parent == NULL)
+				return;
+
+			kernel_driver = udev_device_get_driver(parent);
+			if (kernel_driver == NULL)
+				return;
+		}
+	}
+
 	if (driver == NULL) {
-		const char *drv;
 		unsigned int i;
 
-		drv = udev_device_get_property_value(device, "ID_USB_DRIVER");
-		if (drv == NULL) {
-			drv = udev_device_get_driver(device);
-			if (drv == NULL) {
-				struct udev_device *parent;
-
-				parent = udev_device_get_parent(device);
-				if (parent == NULL)
-					return;
-
-				drv = udev_device_get_driver(parent);
-				if (drv == NULL)
-					return;
-			}
-		}
-
-
-		DBG("%s [%s:%s]", drv, vendor, model);
+		DBG("%s [%s:%s]", kernel_driver, vendor, model);
 
 		if (vendor == NULL || model == NULL)
 			return;
 
 		for (i = 0; vendor_list[i].driver; i++) {
-			if (g_str_equal(vendor_list[i].drv, drv) == FALSE)
+			if (g_strcmp0(vendor_list[i].drv, kernel_driver))
 				continue;
 
 			if (vendor_list[i].vid) {
@@ -1894,8 +2071,8 @@ static void check_usb_device(struct udev_device *device)
 			return;
 	}
 
-	add_device(syspath, devname, driver, vendor, model, device,
-			MODEM_TYPE_USB);
+	add_device(syspath, devname, driver, vendor, model, MODEM_TYPE_USB,
+			device, kernel_driver);
 }
 
 static const struct {
@@ -1911,7 +2088,8 @@ static const struct {
 static void check_pci_device(struct udev_device *device)
 {
 	const char *syspath, *devname, *driver;
-	const char *vendor = NULL, *model = NULL, *drv = NULL;
+	const char *vendor = NULL, *model = NULL;
+	const char *kernel_driver;
 	unsigned int i;
 
 	syspath = udev_device_get_syspath(device);
@@ -1923,14 +2101,14 @@ static void check_pci_device(struct udev_device *device)
 	vendor = udev_device_get_sysattr_value(device, "vendor");
 	model = udev_device_get_sysattr_value(device, "device");
 	driver = udev_device_get_property_value(device, "OFONO_DRIVER");
-	drv = udev_device_get_property_value(device, "DRIVER");
-	DBG("%s [%s:%s]", drv, vendor, model);
+	kernel_driver = udev_device_get_property_value(device, "DRIVER");
+	DBG("%s [%s:%s]", kernel_driver, vendor, model);
 
-	if (vendor == NULL || model == NULL || drv == NULL)
+	if (vendor == NULL || model == NULL || kernel_driver == NULL)
 		return;
 
 	for (i = 0; pci_driver_list[i].driver; i++) {
-		if (g_str_equal(pci_driver_list[i].drv, drv) == FALSE)
+		if (g_strcmp0(pci_driver_list[i].drv, kernel_driver))
 			continue;
 
 		if (pci_driver_list[i].vid) {
@@ -1949,9 +2127,10 @@ static void check_pci_device(struct udev_device *device)
 	if (driver == NULL)
 		return;
 
-	add_device(syspath, devname, driver, vendor, model, device,
-			MODEM_TYPE_PCIE);
+	add_device(syspath, devname, driver, vendor, model, MODEM_TYPE_PCIE,
+			device, kernel_driver);
 }
+
 static void check_device(struct udev_device *device)
 {
 	const char *bus;
