@@ -20,6 +20,7 @@
  */
 
 #include <glib.h>
+#include <ell/ell.h>
 
 #define OFONO_API_SUBJECT_TO_CHANGE
 
@@ -131,7 +132,7 @@ enum ofono_atom_type {
 	OFONO_ATOM_TYPE_SSN,
 	OFONO_ATOM_TYPE_MESSAGE_WAITING,
 	OFONO_ATOM_TYPE_CBS,
-	OFONO_ATOM_TYPES_CALL_VOLUME,
+	OFONO_ATOM_TYPE_CALL_VOLUME,
 	OFONO_ATOM_TYPE_GPRS,
 	OFONO_ATOM_TYPE_GPRS_CONTEXT,
 	OFONO_ATOM_TYPE_RADIO_SETTINGS,
@@ -216,6 +217,42 @@ gboolean __ofono_modem_remove_atom_watch(struct ofono_modem *modem,
 
 void __ofono_atom_free(struct ofono_atom *atom);
 
+const void *__ofono_atom_driver_builtin_find(const char *name,
+				const struct ofono_atom_driver_desc *start,
+				const struct ofono_atom_driver_desc *stop);
+
+#define OFONO_DEFINE_ATOM_CREATE(type, atom_type, ...)			\
+extern struct ofono_atom_driver_desc __start___ ## type[];		\
+extern struct ofono_atom_driver_desc __stop___ ## type[];		\
+									\
+struct ofono_ ## type *ofono_ ## type ##_create(			\
+				struct ofono_modem *modem,		\
+				unsigned int vendor, const char *driver,\
+				void *data)				\
+{									\
+	const struct ofono_ ## type ## _driver *drv =			\
+		__ofono_atom_driver_builtin_find(driver,		\
+				__start___ ## type,			\
+				__stop___ ## type);			\
+	struct ofono_ ## type *atom;					\
+									\
+	if (!drv || !drv->probe)					\
+		return NULL;						\
+									\
+	atom = g_new0(struct ofono_ ##type, 1);				\
+	atom->atom = __ofono_modem_add_atom(modem, atom_type,		\
+				type ##_remove, atom);			\
+	__VA_ARGS__							\
+									\
+	if (drv->probe(atom, vendor, data) < 0)	{			\
+		ofono_ ## type ##_remove(atom);				\
+		return NULL;						\
+	}								\
+									\
+	atom->driver = drv;						\
+	return atom;							\
+}
+
 typedef void (*ofono_modemwatch_cb_t)(struct ofono_modem *modem,
 					gboolean added, void *data);
 void __ofono_modemwatch_init(void);
@@ -387,9 +424,8 @@ ofono_bool_t __ofono_is_valid_sim_pin(const char *pin,
 
 ofono_bool_t __ofono_is_valid_net_pin(const char *pin);
 
-void __ofono_sim_refresh(struct ofono_sim *sim, GSList *file_list,
-				ofono_bool_t full_file_change,
-				ofono_bool_t naa_init);
+void __ofono_sim_refresh(struct ofono_sim *sim, struct l_queue *file_list,
+				bool full_file_change, bool naa_init);
 
 void __ofono_sim_recheck_pin(struct ofono_sim *sim);
 
@@ -418,6 +454,7 @@ unsigned char *__ofono_sim_session_get_aid(
 		struct ofono_sim_aid_session *session);
 
 const char *__ofono_sim_get_impi(struct ofono_sim *sim);
+void __ofono_sim_clear_cached_pins(struct ofono_sim *sim);
 
 #include <ofono/stk.h>
 
@@ -526,8 +563,6 @@ void __ofono_nettime_probe_drivers(struct ofono_modem *modem);
 void __ofono_nettime_info_received(struct ofono_modem *modem,
 					struct ofono_network_time *info);
 
-#include <ofono/cdma-voicecall.h>
-#include <ofono/cdma-connman.h>
 #include <ofono/sim-auth.h>
 
 #include <ofono/gprs-provision.h>
@@ -553,11 +588,6 @@ void __ofono_emulator_slc_condition(struct ofono_emulator *em,
 					enum ofono_emulator_slc_condition cond);
 
 #include <ofono/gnss.h>
-#include <ofono/cdma-sms.h>
-#include <ofono/cdma-netreg.h>
-
-#include <ofono/cdma-provision.h>
-ofono_bool_t __ofono_cdma_provision_get_name(const char *sid, char **name);
 
 #include <ofono/private-network.h>
 
